# -*- coding: UTF-8 -*-
#!/usr/bin/env python

"""
Tests unitaires de cohérence de la matrice de flux

"""

__revision__ = "$Id: unittest_matrix.py,v 1.2 2004/02/13 13:23:15 adrien Exp $"

import autopath
try:
    from logilab.common.testlib import TestCase, unittest_main as main
except:
    from unittest import TestCase, main

from era.compat import sorted, set
from era.noyau.erreurs import *
from era.noyau.fwobjects import *
from era.noyau.models import *
    
APP = 'editeur'
DIR = 'i18n'
import locale, gettext
# locale.setlocale (locale.LC_ALL, '')
gettext.bindtextdomain (APP, DIR)
gettext.textdomain (APP)
# installons la fonction _() dans les builtins
gettext.install (APP, DIR, unicode=1)


FLUX_ZONE_A = """[Flux : [zB <===> zA], Flux : [zC <===> zA]]"""

FLUX_ZONE_C_REMOVED = """[Flux : [zB <===> zA]]"""

class TestMatrice(TestCase) :

    def setUp(self):
        self._init_zones()
        self._init_services()
        self._init_extremites()
        self._init_directives()
        self._init_matrice()
        
    def _init_zones(self):
        zoneA = Zone('zA', 30, '127.0.0.1')
        zoneB = Zone('zB', 31, '127.0.0.1')
        zoneC = Zone('zC', 32, '127.0.0.1')
        
        self.zones = {'zoneA': zoneA,
                      'zoneB': zoneB,
                      'zoneC': zoneC
                      }
        
    def _init_services(self):
        s1 = Service("serv1", "TCP", [80], "serv1", "libelle_Internet")
        s2 = Service("serv2", "TCP", [480], "serv2", "libelle2")
        s3 = Service("Youpi", "UDP", [4513], "serv3", "libelle3")
        s4 = Service("echo-request", "ICMP", [0], "echo-request", "règle icmp echo-request")
        sg1 = ServiceGroupe("Groupe1", "test de groupe de services",[s1,s2])
        self.services = {"serv1" : s1, "serv2" : s2, "Youpi" : s3, "icmp" : s4}
        self.liste_services = [s1,s2,s3,s4]
        self.groupe_services = {"grp1" : sg1}

    def _init_extremites(self):
        e1 = Extremite(self.zones['zoneA'],"extr1", "libelle1", ["10.0.0.1"], "255.255.255.255")
        e2 = Extremite(self.zones['zoneA'],"extr2", "libelle2", ["10.0.0.2"], "255.255.255.255")
        e3 = Extremite(self.zones['zoneB'],"extr3", "libelle3", ["10.0.0.3"], "255.255.255.255")
        e4 = Extremite(self.zones['zoneC'],"extr4", "libelle4", ["10.0.0.4"], "255.255.255.255")
        self.extremites = {"extr1" : e1,"extr2" : e2,"extr3" : e3,"extr4" : e4}
        self.groupe_extremites = {'grp1' : [e1,e2,e3], 'grp2' : [e2,e3,e4]}

    def _init_directives(self):
        self.d0 = Directive([self.extremites['extr1']], [self.extremites['extr3']],
                            self.services['serv1'], ACTION_DENY, 0)
        
        self.d1 = Directive([self.extremites['extr3']],[self.extremites['extr1']], 
                            self.services['serv1'], ACTION_ALLOW, 0)

        self.directives = [self.d0, self.d1]


    def _init_matrice(self) :
        self.matrice = MatrixModel()

        for zone in self.zones :
            self.matrice.add_zone(self.zones[zone])
        self.matrice.flux[0].up_directives_store.add_directive(self.d0)
        self.matrice.flux[0].up_directives_store.add_directive(self.d1)


    def test_flux(self) :
        """Test sur les flux reliés à la zone A
        """
        self.assertEqual(str(self.matrice.get_flux_list(self.zones["zoneA"])), FLUX_ZONE_A)

    def test_del_zone(self) :
        """Test de réorganisation des flux sur la suppression d'une zone dans la matrice
        """
        self.matrice.remove_zone(self.zones["zoneC"])
        self.assertEqual(str(self.matrice.get_flux_list(self.zones["zoneA"])), FLUX_ZONE_C_REMOVED)


    def test_directive_montante(self) :
        """Test de vérification du positionnement d'une directive dans la matrice
        """
        self.failUnless(self.d0.is_montante())
        self.failIf(self.d1.is_montante())


    def _modify_and_test_instanciation(self, obj):
        for attr in obj.instanciable_attributes:
            initial = getattr(obj, attr)
            setattr(obj, attr, '%%some_variable%%')
            self.failUnless(self.matrice.needs_instanciation())
            setattr(obj, attr, initial)
            # on revérifie juste que tout est remis dans l'ordre
            self.failIf(self.matrice.needs_instanciation())
        

    def test_needs_instanciation(self):
        self.failIf(self.matrice.needs_instanciation())
        self._modify_and_test_instanciation(self.zones['zoneA'])
        self._modify_and_test_instanciation(self.services['serv1'])

    def test_unspecified_vars(self):
        self.assertEquals(self.matrice.unspecified_vars(), set())
        zoneA = self.zones['zoneA']
        zoneA.ip = '%%ip0%%'
        zoneA.network = '%%network0%%'
        self.assertEquals(self.matrice.unspecified_vars(),
                          set(['ip0', 'network0']))
        self.matrice.flux[0].up_directives_store.add_directive(self.d0)
        serv1 = self.services['serv1']
        serv1.protocol = '%%protocol0%%'
        self.assertEquals(self.matrice.unspecified_vars(),
                          set(['ip0', 'network0', 'protocol0']))


    def test_unspecified_vars_extremites(self):
        self.assertEquals(self.matrice.unspecified_vars(), set())
        extr = self.extremites['extr1']
        extr.ip_list[0] = '%%ip1%%'
        extr.netmask = '%%netmask1%%'
        self.assertEquals(self.matrice.unspecified_vars(),
                          set(['ip1', 'netmask1']))
        
    
if __name__ == "__main__":
    main()
    # self.assertEqual(str(self.matrice.get_flux_list(self.zones["zoneA"])), FLUX_ZONE_C_REMOVED)
    # f = open("sortie.txt","w")
    # s = str(self.matrice.get_flux_list(self.zones["zoneA"]))
    # f.write(s) 
    # f.close()
    
