# -*- coding: UTF-8 -*-
"""Module de tests unitaires pour l'API de génération
de règles iptables
"""
import autopath
import sys
import unittest

from era.backend import iptrule
from cStringIO import StringIO

class LStringIO(list):
    """petite classe qui émule write et getvalue
    en utilisant des listes
    """
    write = list.append
    def getvalue(self, join_char = ''):
        return join_char.join(self)


class ParamsTC(unittest.TestCase):
    """Iptable writer TestCase class
    """
    
    def test_ip_options(self):
        """Teste IPOptions (-s / -d)
        """
        param = iptrule.SrcIPParameter('0.0.0.0', '0')
        self.assertEquals(str(param),
                          '-s 0.0.0.0/0')
        
        param = iptrule.DestIPParameter('0.0.0.0')
        self.assertEquals(str(param),
                          '-d 0.0.0.0')

    def test_interface_options(self):
        """Teste InterfaceParameter (-i / -o)
        """
        param = iptrule.InterfaceParameter('eth2', out = False)
        self.assertEquals(str(param),
                          '-i eth2')
        param = iptrule.InterfaceParameter('eth2', out = True)
        self.assertEquals(str(param),
                          '-o eth2')
        param = iptrule.InterfaceParameter('eth2', out = False, inverted = True)
        self.assertEquals(str(param),
                          '-i ! eth2')
        

    def test_icmp_options(self):
        """Teste ICMPParameter
        """
        param = iptrule.ICMPProtocol('source-quench')
        self.assertEquals(str(param),
                          '-p icmp --icmp-type source-quench')
        
        param = iptrule.ICMPProtocol('source-quench', inverted = True)
        self.assertEquals(str(param),
                          '-p icmp --icmp-type ! source-quench')
        

    def test_tcp_options(self):
        """Teste TCPOptions
        """
        param = iptrule.TCPProtocol(dport = '1', sport = '2')
        self.assertEquals(str(param),
                          '-p tcp --dport 1 --sport 2')
        param = iptrule.TCPProtocol(dport = [1,10], sport = '1')
        self.assertEquals(str(param),
                          '-p tcp --dport 1:10 --sport 1')
        param = iptrule.TCPProtocol(dport = 1, sport = 2, flags = ['SYN', 'ACK'])
        self.assertEquals(str(param),
                          '-p tcp --dport 1 --sport 2 --tcp-flags SYN,ACK SYN')
        self.assertRaises(AssertionError, iptrule.TCPProtocol, 1, 2, ['SYNY'])
        

    def test_udp_options(self):
        """Teste UDPOptions
        """
        param = iptrule.UDPProtocol(dport = 1, sport = [2, 10])
        self.assertEquals(str(param),
                          '-p udp --dport 1 --sport 2:10')
        
        
    def test_basic_target(self):
        """Teste les cibles basiques (DROP, ACCEPT, etc.)
        """
        target = iptrule.Target('DROP')
        self.assertEquals(str(target),
                          '-j DROP')
        
        
    def test_redirect_target(self):
        """Teste cible redirect
        """
        target = iptrule.Redirect(2)
        self.assertEquals(str(target),
                          '-j REDIRECT --to-ports 2')
        target = iptrule.Redirect([2,10])
        self.assertEquals(str(target),
                          '-j REDIRECT --to-ports 2-10')
        
        
    def test_redirect_target(self):
        """Teste cible redirect
        """
        target = iptrule.Masquerade(2)
        self.assertEquals(str(target),
                          '-j MASQUERADE --to-ports 2')
        target = iptrule.Masquerade([2,10])
        self.assertEquals(str(target),
                          '-j MASQUERADE --to-ports 2-10')


    def test_log_target(self):
        """Teste cible LOG
        """
        target = iptrule.Log('warn', 'intrusion de :')
        self.assertEquals(str(target),
                          '-j LOG --log-level warn --log-prefix intrusion de :')
        

    def test_ulog_target(self):
        """Teste la cible ULOG
        """
        target = iptrule.ULog(1, 'prefixe', 4, 2)
        self.assertEquals(str(target),
                          '-j ULOG --ulog-nlgroup 1 --ulog-prefix prefixe --ulog-cprange 4 --ulog-qtreshold 2')
        target = iptrule.ULog(1, 'prefixe')
        self.assertEquals(str(target),
                          '-j ULOG --ulog-nlgroup 1 --ulog-prefix prefixe')


    def test_mark_target(self):
        """Teste la cible MARK
        """
        target = iptrule.Mark('marque')
        self.assertEquals(str(target),
                          '-j MARK --set-mark marque')
        

    def test_reject_target(self):
        """Teste la cible REJECT
        """
        target = iptrule.Reject('icmp-net-unreachable')
        self.assertEquals(str(target),
                          '-j REJECT --reject-with icmp-net-unreachable')
        
        self.assertRaises(AssertionError, iptrule.Reject, 'blabla')

    
    def test_tos_target(self):
        """Teste la cible TOS
        """
        target = iptrule.Tos(16)
        self.assertEquals(str(target),
                          '-j TOS --set-tos 16')
        target = iptrule.Tos('Minimize-Delay')
        self.assertEquals(str(target),
                          '-j TOS --set-tos Minimize-Delay')
        
        self.assertRaises(AssertionError, iptrule.Tos, 'blabla')


    def test_nat_target(self):
        """Teste les cibles DNAT / SNAT
        """
        target = iptrule.Dnat([['0.0.0.0', '0.0.0.1'],
                                   '1.1.1.1',
                                   '2.2.2.2'],
                                  [12, [13,15], []])
        self.assertEquals(str(target),
                          '-j DNAT --to-destination 0.0.0.0-0.0.0.1:12 --to-destination 1.1.1.1:13-15 --to-destination 2.2.2.2')

        target = iptrule.Dnat([['0.0.0.0', '0.0.0.1'],
                                   '1.1.1.1'],
                                  [[12,15], []])
        self.assertEquals(str(target),
                          '-j DNAT --to-destination 0.0.0.0-0.0.0.1:12-15 --to-destination 1.1.1.1')

        self.assertRaises(AssertionError, iptrule.Nat, [], [], 'toto')
        


# backward compat
run = unittest.main

if __name__ == '__main__':
    unittest.main()
